package com.supervision.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.lang.Assert;
import cn.hutool.core.lang.UUID;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.supervision.constant.PromptTemplate;
import com.supervision.domain.AgentDialogueLog;
import com.supervision.domain.AgentInfo;
import com.supervision.dto.AgentChatReqDTO;
import com.supervision.dto.AgentDialogueLogDTO;
import com.supervision.dto.AgentInfoDTO;
import com.supervision.dto.ChatResponseDTO;
import com.supervision.service.*;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.ai.chat.messages.AssistantMessage;
import org.springframework.ai.chat.messages.Message;
import org.springframework.ai.chat.messages.UserMessage;
import org.springframework.ai.chat.metadata.ChatResponseMetadata;
import org.springframework.ai.chat.prompt.Prompt;
import org.springframework.stereotype.Service;
import reactor.core.publisher.Flux;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

@Slf4j
@Service
@RequiredArgsConstructor
public class AgentServiceImpl implements AgentService {

    private final AiCallService aiCallService;

    private final AgentInfoService agentInfoService;

    private final AgentDialogueLogService agentDialogueLogService;
    @Override
    public Flux<ChatResponseDTO> streamChat(AgentChatReqDTO agentChatReqDTO) {

        // 查询历史对话（记忆功能）
        List<Message> messages = buildHistoryMessages(agentChatReqDTO.getConversationId(), 10);
        // 构建当前对话
        Message message = buildQueryMessage(agentChatReqDTO.getQuery(), agentChatReqDTO.getAgentCode());
        Assert.notNull(message, "Agent not found for ID: " + agentChatReqDTO.getAgentCode());
        messages.add(message);
        AgentDialogueLogDTO dialogueLogDTO = new AgentDialogueLogDTO(agentChatReqDTO);
        StringBuilder aiResponseBuilder = new StringBuilder();
        Flux<ChatResponseDTO> map = aiCallService.stream(new Prompt(messages))
                .scan(new ThinkTagState(), (state, response) -> {
                    String chunk = response.getResult().getOutput().getText();
                    state.setChatResponse(response);
                    return state.process(chunk);
                })
                .filter(state -> StrUtil.isNotEmpty(state.getFilteredText()))
                .map(response -> {
                    ChatResponseDTO responseDTO = new ChatResponseDTO();
                    responseDTO.setAnswer(response.getChatResponse().getResult().getOutput().getText());
                    responseDTO.setConversationId(StrUtil.isNotEmpty(agentChatReqDTO.getConversationId()) ? agentChatReqDTO.getConversationId() : UUID.fastUUID().toString());
                    ChatResponseMetadata metadata = response.getChatResponse().getMetadata();
                    Boolean done = metadata.get("done");
                    responseDTO.setEvent(done ? "message_end" : "message");
                    dialogueLogDTO.setConversationId(responseDTO.getConversationId());
                    aiResponseBuilder.append(responseDTO.getAnswer());
                    return responseDTO;
                }).doOnComplete(() -> {
                    dialogueLogDTO.setSystemOut(aiResponseBuilder.toString());
                    agentDialogueLogService.save(dialogueLogDTO.toDialogueLog());
                }).doOnError(e -> {
                    log.error("Error during AI chat stream: {}", e.getMessage());
                    dialogueLogDTO.setSystemOut(aiResponseBuilder.toString());
                    dialogueLogDTO.setAnswerType(1);
                    agentDialogueLogService.save(dialogueLogDTO.toDialogueLog());
                });
        return map;
    }

    @Override
    public IPage<AgentInfoDTO> pageList(Integer page, Integer pageSize) {
        IPage<AgentInfo> paged = agentInfoService.page(Page.of(page, pageSize));
        return paged.convert(AgentInfoDTO::new);
    }

    private Message buildQueryMessage(String query,String agentCode) {
        if (StrUtil.equals(agentCode, "1")) {
            String template = PromptTemplate.GENERAL_INDUSTRY_TEMPLATE;
            return new UserMessage(StrUtil.format(template, Map.of("query",query)));
        }else if (StrUtil.equals(agentCode, "2")) {
            String template = PromptTemplate.HEART_GUIDE_TEMPLATE;
            return new UserMessage(StrUtil.format(template, Map.of("query",query)));
        }
        return null;
    }
    private List<Message> buildHistoryMessages(String conversationId,int historySize) {
        List<Message> messages = new ArrayList<>();
        if (StrUtil.isEmpty(conversationId)){
            return messages;
        }
        IPage<AgentDialogueLog> dialogueLogs = agentDialogueLogService.pageList(Page.of(1, historySize), conversationId);
        dialogueLogs.getRecords().sort((o1, o2) -> - o2.getCreateTime().compareTo(o1.getCreateTime()));
        if (CollUtil.isNotEmpty(dialogueLogs.getRecords())) {
            for (AgentDialogueLog dialogueLog : dialogueLogs.getRecords()) {
                if (dialogueLog.getAnswerType() == 1){
                    continue;
                }
                if (StrUtil.isNotEmpty(dialogueLog.getUserInput()) && historySize > 0) {
                    messages.add(new UserMessage(dialogueLog.getUserInput()));
                    String result = dialogueLog.getSystemOut().replaceAll("(?is)<think\\b[^>]*>(.*?)</think>", "").trim();
                    messages.add(new AssistantMessage(result));
                    historySize--;
                }
            }
        }
        return messages;
    }
}
